# -*- coding: UTF-8 -*-

__author__ = 'romain.jouin@gmail.com'


import  time


def analysis_initialisation(path_to_the_main_table, colonne_de_maille, col_y, sep=";"):
    """
    Create a DF from the given path, group it by the given column,
    and create another DF indexed by the col value, and having a col with the groups generated by the groupby object.
    Replace [,] by [.] on all cells and try then to convert numeric values (calling convert_objects)

    :param path_to_the_main_table: string (path to a csv file)
    :param colonne_de_maille: string (column containing the values for mesh analysis)
    :param sep: string (cv separator)
    :param dtype: array (experimental : trying to path a dtype to the df :-s )
    :return: a groupby object and a DF (containing one column and indexed with the value of this column)
    """
    import pandas as pd
    import numpy as np
    """
        0) verify parameters
    """
    assert is_file(path_to_the_main_table), "%s is not a file" % (path_to_the_main_table)
    print("path_to_the_main_table = ", path_to_the_main_table)
    """
        1) load Main table with the index being the "colonne de maille"
    """

    main_table = get_dataframe(path_to_the_main_table, sep, colonne_de_maille)
    main_table[colonne_de_maille] = main_table.index
    """
        2) Group the main table by the "colonne de maille" (which is now the index)
    """
    print("Grouping by index")
    grouped_o = main_table.groupby(main_table.index)
    """
        3) Extract Y
    """
    print("Extracting Y")
    y = pd.DataFrame()
    y[col_y] = grouped_o[col_y].sum()
    y.index = list(grouped_o.groups.keys())
    """
        4) Create aggregation DataFrame:
                index = colonne de maille
                une colonne avec la colonne de maille
                une colonne avec Y

    """
    print("Creating aggregation dataframe")
    aggregation = pd.DataFrame()
    aggregation[colonne_de_maille] = list(grouped_o.groups.keys())
    aggregation.index = list(grouped_o.groups.keys())
    aggregation["y"] = y[col_y]
    aggregation[col_y] = y[col_y]

    try:
        print("Initialisation finished, returning 3 df and a group : main = %s aggregation = %s y= %s" % (
        main_table.shape, aggregation.shape, y.shape))
    except:
        pass
    """
    try:
        #do_pickle(main_table, "main_df")
        #do_pickle(grouped_o, "grouped_o")
        #do_pickle(aggregation, "aggregation")
        #do_pickle(y, "y")
    except:
        print "Error in piclking"
    """

    return main_table, grouped_o, aggregation, y
def auto_aggregate_with_specificities(objet, input_dataframe, collonne_de_maille, specific, aggregation, sep=";"):
    """
    Create a df from the givent filepath path, and call auto_analysis on the colonne de maille.
    Plus call count_specificites on the specific array. Both aggregating on the aggregation df.

    :param objet: string (Just to display on screen)
    :param input_dataframe:  pandas.dataFrame (DataFrame from which to gruop and do auto analysis )
    :param collonne_de_maille: string (column on which to apply the group for auto analysis)
    :param specific: array (columns where to apply count_specifities)
    :param aggregation: pandas.DataFrame (dataframe to complete with the aggregations)
    :return: pandas.DataFrame (completed aggregation passed as parameter)
    """
    """
        ALGORITHM
    """
    title(objet)

    aggregation, left_aside = auto_analysis(input_dataframe, collonne_de_maille, aggregation)
    aggregation = count_specificities(input_dataframe, collonne_de_maille, specific, aggregation)
    left_aside = left_aside.difference(set(specific))

    """
        SUMMARY
    """

    print("%s : %s col left aside = [ %s ]" % (objet, len(left_aside), " ; ".join(list(left_aside))))
    print("aggregation shape = :  ", aggregation.shape)
    title("%s - FIN" % (objet))
    return aggregation
def automatic_df_treatment(main_df, specific, aggregation_df):
    modalities, binaries, continuous, dates = select_columns_based_on_names(main_df)
    auto = modalities + binaries + continuous + dates

    left_aside = set(main_df.columns) - set(auto) - set(specific)

    for special in specific:
        aggregation[special] = gpd_df[special].count()

    for modality in modalities:
        aggregation["mod_of_%s" % (modality)] = gpd_df[modality].agg(lambda x: x.value_counts().index[0])
        aggregation["prct_of_mod_of_%s" % (modality)] = gpd_df[modality].agg(
            lambda x: x.value_counts().values[0] / sum(x.value_counts().values))

    for binary in binaries:
        main_df[continuum] = main_df[continuum].convert_objects(convert_numeric=True)
        aggregation["pc_of_%s" % (binary)] = gpd_df[binary].sum()

    for continuum in continuous:
        main_df[continuum] = main_df[continuum].convert_objects(convert_numeric=True)
        aggregation["min_of_%s" % (continuum)] = gpd_df[continuum].min()
        aggregation["median_of_%s" % (continuum)] = gpd_df[continuum].median()
        aggregation["max_of_%s" % (continuum)] = gpd_df[continuum].max()
def auto_analysis(big_dataframe, group_key, aggregation_df):
    """
    Calcul des valeurs selon le type de colonne dans une df (via select_columns_based_on_names)
    """
    import time
    import pandas as pd
    assert group_key in big_dataframe.columns, "%s not in DF columns (%s)" % (
    group_key, "\n".join(big_dataframe.columns))
    modalities, binaries, continuous, dates = select_columns_based_on_names(big_dataframe)

    gpd_df = big_dataframe.groupby(group_key)
    gpd_df.index = big_dataframe[group_key]
    start1 = title2("Auto Analysis")

    """ ============================
                modalities
        ============================"""
    for modality in modalities:
        print(modality)
        mode = gpd_df[modality].agg([("mod_of_%s" % (modality), lambda x: x.value_counts().index[0] if len(
            x.value_counts().index) > 0 else False)])
        mode_pct = gpd_df[modality].agg([("pct_of_mod_of_%s" % (modality),
                                          lambda x: x.value_counts().values[0] / sum(x.value_counts().values) if len(
                                              x.value_counts().index) > 0 else False)])
        nb = gpd_df[modality].agg([("nb_of_%s" % (modality), lambda x: len(x.value_counts()))])

        aggregation_df = pd.merge(aggregation_df, mode, left_index=True, right_index=True, how='left')
        aggregation_df = pd.merge(aggregation_df, mode_pct, left_index=True, right_index=True, how='left')
        aggregation_df = pd.merge(aggregation_df, nb, left_index=True, right_index=True, how='left')

    """ ============================
                Binaries
        ============================"""
    for binary in binaries:
        print(binary)
        sum_binary = gpd_df[binary].agg([("sum_of_%s" % (binary), sum)])
        aggregation_df = pd.merge(aggregation_df, sum_binary, left_index=True, right_index=True, how='left')
    """ ============================
                continous
        ============================"""
    for continuum in continuous:
        print(continuum)
        try:
            aggregation_df["min_of_%s" % (continuum)] = gpd_df[continuum].min()
            aggregation_df["median_of_%s" % (continuum)] = gpd_df[continuum].median()
            aggregation_df["max_of_%s" % (continuum)] = gpd_df[continuum].max()
        except pd.core.groupby.DataError as e:
            print("ATTENTION => %s is not numerical ! Trying to force conversion" % (continuum))
            try:
                big_dataframe[continuum] = big_dataframe[continuum].convert_objects(convert_numeric=True)
                gpd_df = big_dataframe.groupby(group_key)

                aggregation_df["min_of_%s" % (continuum)] = gpd_df[continuum].min()
                aggregation_df["median_of_%s" % (continuum)] = gpd_df[continuum].median()
                aggregation_df["max_of_%s" % (continuum)] = gpd_df[continuum].max()
            except Exception as e:
                print(e)

    """ ============================
                End
        ============================"""
    auto_columns = modalities + binaries + continuous + dates
    left_aside = set(big_dataframe.columns) - set(auto_columns)
    return aggregation_df, left_aside
def add_quantile(df, nom_col):
    """
    cree une colonne quantilisee dans la df de la colonne passee en param
    """
    # Param
    col_quantile = df[nom_col]
    new_col = "quantile_{0}".format(nom_col)
    bins = [0, 0.05, 0.25, 0.5, 0.75, 0.95, 1]
    # algo
    quantiles = col_quantile.quantile(bins)
    df[new_col] = pd.cut(col_quantile, quantiles)
    return df
def count_specificities(input_df, gp_col, specific_cols, aggregation_df):
    """

    """
    import time
    import pandas as pd
    for x in specific_cols:
        assert x in input_df.columns, "col %s not in df's columns (%s) " % (x, input_df.columns)

    gpd_df = input_df.groupby(gp_col)

    start = time.time()
    title2("Specificity")
    for modality in specific_cols:
        print(modality)
        mode = gpd_df[modality].agg([("mod_of_%s" % (modality), lambda x: x.value_counts().index[0] if len(
            x.value_counts().index) > 0 else False)])
        mode_pct = gpd_df[modality].agg([("pct_of_mod_of_%s" % (modality),
                                          lambda x: x.value_counts().values[0] / sum(x.value_counts().values) if len(
                                              x.value_counts().index) > 0 else False)])
        nb = gpd_df[modality].agg([("nb_of_%s" % (modality), lambda x: len(x.value_counts()))])

        aggregation_df = pd.merge(aggregation_df, mode, left_index=True, right_index=True, how='left')
        aggregation_df = pd.merge(aggregation_df, mode_pct, left_index=True, right_index=True, how='left')
        aggregation_df = pd.merge(aggregation_df, nb, left_index=True, right_index=True, how='left')

    delay = time.time() - start
    print("%s \tspecific in %.2f sec (%.2f / loop)" % (len(specific_cols), delay, delay / max(len(specific_cols), 1)))
    return aggregation_df
def create_df_and_auto_aggregate(objet, filepath, collonne_de_maille, specific, aggregation, sep=";"):
    """
    Create a df from the givent filepath path, and call auto_analysis on the colonne de maille.
    Plus call count_specificites on the specific array. Both aggregating on the aggregation df.

    :param objet: string (Just to display on screen)
    :param filepath:  string (file path to the data for the csv)
    :param collonne_de_maille: string (column on which to apply the group for auto analysis)
    :param specific: array (columns where to apply count_specifities)
    :param aggregation: pandas.DataFrame (dataframe to complete with the aggregations)
    :return: pandas.DataFrame (completed aggregation passed as parameter)
    """
    """
        ALGORITHM
    """
    title(objet)

    df = get_dataframe(filepath, sep, collonne_de_maille)

    aggregation, left_aside = auto_analysis(df, collonne_de_maille, aggregation)
    aggregation = count_specificities(df, collonne_de_maille, specific, aggregation)
    left_aside = left_aside.difference(set(specific))

    """
        SUMMARY
    """

    print("%s : %s col left aside = [ %s ]" % (objet, len(left_aside), " ; ".join(list(left_aside))))
    print("aggregation shape = :  ", aggregation.shape)
    title("%s - FIN" % (objet))
    del df
    return aggregation
def calculate_nunique(df):
    """
    Apply nunique() on each column of the dataframe
    Parameters:
        df: dataframe to be analysed
            pandas dataframe
    Return:
        Panda dataframe

    """
    import pandas as pd
    r = {col: df[col].nunique() for col in df.columns}
    return pd.DataFrame(r, index=["nunique"]).T
def calculate_pc_unique(df):
    """
    Apply nunique()/shape[0] on each column of the dataframe
    Parameters:
        df: dataframe to be analyse
            pandas dataframe
    Return:
        Panda dataframe

    """
    import pandas as pd
    r = {col: df[col].nunique() / df.shape[0] for col in df.columns}
    return pd.DataFrame(r, index=["pct_unique"]).T
def calculate_unique(df):
    """
    Apply nunique()/shape[0] on each column of the dataframe
    Parameters:
        df: dataframe to be analyse
            pandas dataframe
    Return:
        Panda dataframe

    """
    import pandas as pd
    nunique = {col: df[col].nunique() for col in df.columns}
    pct_unique = {col: df[col].nunique() / df.shape[0] for col in df.columns}
    return pd.DataFrame([nunique, pct_unique], index=["nb_unique", "pct_unique"]).T
def calculate_unique_null_not_null(df):
    import pandas as pd
    df1 = calculate_unique(df)
    df2 = calculate_nulls(df)
    return pd.concat([df1, df2], axis=1)
def calculate_nulls(df):
    import pandas as pd
    nb_null = {col: sum(df[col].isnull()) / df.shape[0] for col in df.columns}
    not_null = {col: 1 - (sum(df[col].isnull())) / df.shape[0] for col in df.columns}
    return pd.DataFrame([nb_null, not_null], index=["pct_null", "pct_not_null"]).T
def columns_examples(df, random=True, n_sample=3):
    """
    Retourne une matrice avec une analyse des colonnes : nom, type, et examples
    Parameters:
        df : Dataframe to be analysed
            pd.Dataframee
        random : should we take random rows (using df.sample)
            boolean
        n_samples : nb of example to give
            int
    Return:
        pandas Dataframe
    """
    from collections import OrderedDict
    import pandas as pd
    test = df[:n_sample].copy()
    if random:
        test = df.sample(n_sample).copy()
    types = test.dtypes
    to_print = []
    for i, col in enumerate(test.columns):
        p = OrderedDict()
        p["col"] = col
        p["type"] = types[i]
        for i, v in enumerate(test[col].values):
            p["examp_%s" % i] = str(v)

        to_print.append(p)

    return pd.DataFrame(to_print)
def convert_serie_to_df(serie, col_handling_index):
    """
    Create a DF with two columns : one with the serie contained in the given serie,
    and a 2nd one  containing the index of the serie as column values (this col has the same value as the index of the nex df)
    :param serie: Pandas Serie to be converted
    :param col_handling_index: Name to give to the created column
    :return: Panda Dataframe
    """
    import pandas as pd
    assert isinstance(serie, pd.Series)
    df = pd.DataFrame(serie)
    df[col_handling_index]=serie.index
    return df
def convert_col_to_date(df, column):
    import pandas as pd
    assert isinstance(df, pd.DataFrame), "The given df is not a Dataframe, is is a [%s]."%(type(df))
    assert column in df.columns, "[%s] is not in Datframe columns (%s)"%(column, df.columns)
    from dateutil.parser import parse
    df[column] = [parse(x) for x in df[column]]
    return df
def clean_string(string):
    string.replace("é", "e")
    string.replace("è", "e")
    string.replace(" ", "_")

    a = [x.lower()  for x in string if x.lower() in "_azertyuiopqsdfghjklmwxcvbn1234567890".lower()]

    return ''.join(a)
def clean_string_array(array):

    return [ clean_string(x) for x in array ]
def do_pickle(df, x):
    pickle_dir_path = "/Users/nouveau/Documents/memorandum/randstad/HEC/pickles/tout_with_small_indicateurs/"
    try:
        do_pickle_in_path(df, x, pickle_dir_path)
    except:
        default = "/Users/nouveau/pickles"
        print("Attention !! \n |-> pbm picklin in [%s] \n |----> will try in [%s] "%(pickle_dir_path , default ))
        do_pickle_in_path(df, x, default )
def do_pickle_in_path(df, x, pickle_dir_path):
    import os
    import pickle
    print("Going to Write down a dataframe %s"%(x))
    filename = "%s_%s"%(time.time(), x)
    save_path = os.path.join(pickle_dir_path, filename)
    df.to_csv(save_path, sep=";")
    print("%s written done in %s in [%s]"%(x, get_file_name(save_path) , pickle_dir_path))
def deal_with_continuous_column(database, col_to_group_by, col_to_analyse, fill_na_with=0):
    """
    Calculate some aggregate over a column, trying to convert it from string to in (values after dot are lost).
    :param database:
    :param col_to_group_by:
    :param col_to_analyse:
    :param fill_na_with:
    :return:
    """
    import pandas as pd
    assert isinstance(database, pd.DataFrame), "database is not a panda dataframe, it is : %s"%(type(database))
    assert isinstance(col_to_group_by, str), "col_to_group_by is not a str, it is : %s"%(type(col_to_group_by))
    assert isinstance(col_to_analyse, str), "col_to_analyse is not a str, it is : %s"%(type(col_to_analyse))
    assert isinstance(fill_na_with, int), "fill_na_with is not a int, it is : %s"%(type(fill_na_with))
    assert col_to_group_by in database.columns,"col_to_group_by (%s) is not in database's cols :[%s] "%(col_to_group_by , database.columns)
    assert col_to_analyse in database.columns,"col_to_analyse (%s) is not in database's cols :[%s] "%(col_to_analyse , database.columns)


    print("in")
    try:
        gp = database[col_to_analyse].groupby(database[col_to_group_by]).mean()
        gp = database[col_to_analyse].groupby(database[col_to_group_by])
    except Exception as e:
        try:
            # the data is not recognise as numeric
            # we try to convert it (from string)

            v0 = database[col_to_analyse].values
            v0 = [v.strip().replace(" ", "") for v in v0]
            try     : v1 = [int(v[:v.find(",")]) for v in v0]
            except  : v1 = [int(v[:v.find(".")]) for v in v0]

            i = database[col_to_analyse].index
            s = pd.Series(v1, index=i)
            gp = s.groupby(database[col_to_group_by])
        except Exception as e :
            print("deal_with_continuous_column : %s "%(e))


    gp_mean = gp.mean()
    gp_sum  = gp.sum()
    gp_max  = gp.max()
    gp_min  = gp.min()
    gp_std  = gp.std()


    sum_name    = "sum_of_%s"%(col_to_analyse)
    mean_name   = "mean_of_%s"%(col_to_analyse)
    max_name    =  "max_of_%s"%(col_to_analyse)
    min_name    =  "min_of_%s"%(col_to_analyse)
    std_name    =  "std_of_%s"%(col_to_analyse)

    gp_sum.name = sum_name
    gp_mean.name = mean_name
    gp_max.name  =  max_name
    gp_min.name  =  min_name
    gp_std.name  =  std_name

    df_sum = convert_serie_to_df(gp_sum, col_to_group_by)
    df_mean = convert_serie_to_df(gp_mean, col_to_group_by)
    df_max  = convert_serie_to_df(gp_max , col_to_group_by)
    df_min  = convert_serie_to_df(gp_min , col_to_group_by)
    df_std  = convert_serie_to_df(gp_std , col_to_group_by)

    df = merge_df(df_mean, df_std)
    df = merge_df(df, df_sum)
    df = merge_df(df, df_min)
    df = merge_df(df, df_max)

    return df
def get_merge(pickle_dir):
    import pickle
    import pandas as pd
    assert is_dir(pickle_dir)

    pickle_paths = get_filepaths(pickle_dir, ".pkl")
    dfs = []
    for pickle_path in pickle_paths:
        print(pickle_path)
        df = pickle.load(open(pickle_path, 'rb'))
        assert isinstance(df, pd.DataFrame)
        dfs.append(df)

    return merge_df_array(dfs)
def get_dataframe(path, sep=";", colonne_de_maille=False, error_bad_lines=False):
    """
    Call the pd.read_csv on the given path.
    :param path: string (complete path to a csv file)
    :param sep: string (csv separator)
    :param error_bad_lines: Boolean
    :return: pd.DataFrame
    """
    import pandas as pd

    assert is_file(path),"%s is not a path to a file"%(path)
    print("Loading %s"%(path))

    df          = pd.read_csv(path, sep=sep,  error_bad_lines=error_bad_lines)
    df.columns  = [clean_string(x).upper() for x in df.columns]
    df          = df.applymap(try_to_transform_string_to_float )

    if colonne_de_maille:
        assert colonne_de_maille in df.columns, "%s not in DF columns (%s)."%(colonne_de_maille, df.columns)
        df.index = df[colonne_de_maille]
    print("Datframe loaded ")
    #do_pickle(df, "df_with_%s_as_col_de_maille"%(colonne_de_maille))
    return df
def example_lecture_csv_by_pandas():
    if path.endswith('histo_permanents_agence_randstad_perf_unite.csv'):
        delimiter = useful.find_delimiter(path)
        dtype = {"MAT_RH": str,
                 "MAT_RHPI": str,
                 "C_SEXE": str,
                 "FTE_PAYE": float,
                 "pourc_ana_defaut": float,
                 "pourc_ana_1": float}

        converters = {"FTE_PAYE": extra_pandas.enforce_float_for,
                      "pourc_ana_defaut": extra_pandas.enforce_float_for,
                      "pourc_ana_1": extra_pandas.enforce_float_for,
                      "% analytique 2": extra_pandas.enforce_float_for,
                      "% analytique 3": extra_pandas.enforce_float_for,
                      "% analytique 4": extra_pandas.enforce_float_for,
                      "% analytique 5": extra_pandas.enforce_float_for}
        parse_dates = [0, 9, 10]

        def time_converter(x):
            if x == 'nan':
                return None
            else:
                try:
                    return datetime.datetime.strptime(x, "%d/%m/%Y")
                except:
                    return datetime.datetime.strptime(x, "%Y%m")

        df = pd.read_csv(path,
                         nrows=nrows,
                         delimiter=delimiter,
                         parse_dates=parse_dates,
                         date_parser=time_converter,
                         dtype=dtype,
                         converters=converters)
        return df
def enforce_float_for(x):
    if isinstance(x, float):
        return x
    try:
        return float(x)
    except:
        if isinstance(x, str):
            try:
                return float(x.replace(" ", "").replace(",", "."))
            except Exception as e:
                print(e)
                return None
def enforce_float(df, column):
    df[column].apply(enforce_float_for)
def enforce_date(df, date_colonne, date_format="", default_date="01/01/2020", default_format="%d/%m/%Y", debug=False):
    """
        call datetime.strptime on the date_colonne of the given dataframe.
    """
    import pandas as pd
    import datetime
    assert isinstance(df, pd.DataFrame), "df is not a Dataframe, it is a [%s]" % (type(df))
    assert date_colonne in df.columns, "[%s] not in [%s]" % (date_colonne, df.columns)
    formats = [date_format,
               "%d-%M-%y",
               "%d-%M-%Y",
               "%d.%m.%y",
               "%d-%m-%y",
               "%d-%m-%Y",
               "%d/%m/%y",
               "%d%b%Y:00:00:00",
               "%d-%m-%Y",
               "%Y/%m/%d 00:00:00",
               "%Y-%m-%d 00:00:00"]
    erreurs = ["31DEC9999", "0986/09/14", "986-09-14 00:00:00"]

    for format_ in formats:
        try:
            df[date_colonne] = df[df[date_colonne].notnull()][date_colonne].apply(
                lambda x: test_date(x, format_, erreurs))
            break
        except Exception as e:
            pass

    date_mini = datetime.datetime.strptime("01-01-1900", "%d-%m-%Y")
    date_maxi = datetime.datetime.strptime(default_date, default_format)

    print("nb de nan apres2 = ", df[date_colonne].isnull().sum())
    df[date_colonne] = df[date_colonne].fillna(date_maxi)
    print("nb de nan apres3 = ", df[date_colonne].isnull().sum())

    df.loc[df[df[date_colonne] < date_mini].index, date_colonne] = date_mini
    df.loc[df[df[date_colonne] > date_maxi].index, date_colonne] = date_maxi

    if debug: print("impossible d'enforcer la colonne [%s] en date'" % date_colonne)
    return False
def enforce_float_to(x, debug=False):
    """
        Try to transform an input (str or float) in float.
        For str inputs, delete blanks and change ',' by '.' before attempting conversion.

        :return: float or exception
    """

    to_return = ""
    if isinstance(x, int):
        return x
    if isinstance(x, float):
        return x

    try:
        return float(x)
    except Exception as e:
        if isinstance(x, str):
            try:
                return int(x.replace(" ", ""))
            except:
                try:

                    return float(x.replace(" ", "").replace(",", "."))
                except Exception as e:
                    r = []
                    for c in x:
                        if c in "1234567890.":
                            r.append(c)
                        else:
                            raise Exception("Found a car not in [1234567890.]")
                    r = ''.join(r)
                    try:
                        return int(r)
                    except:
                        return float(r)
        else:
            if debug: print("Impossible to enforce [%s] (type=%s)en float.Error = [%s]" % (x, type(x), e))
            raise Exception

    return to_return
def enforce_float(df, column, debug=False):
    """
    Call unary function 'enforce_float' over a DataFrame.
    """
    try:
        df[column].apply(lambda x: enforce_float_to(x))
        if debug: print("%s transformed in float" % column)
    except Exception as e:
        if debug: print("impossible de transformer %s transformed in float" % column)
        print(e)
def find_mod(df, col_name, return_first_mode=True):
    """
    Return the mod of a column in a DataFrame
    :param df: Pandas dataframe
    :param col_name: string : columns from which we want the mode
    :param return_first_mode: bool if there are several modes, return all or not
    :return: array
    """
    import pandas as pd
    assert isinstance(df, pd.DataFrame), "df is not a DataFrame"
    assert col_name in df.columns, "Column is not in DataFrame"

    modes = df[col_name].mode().values
    if len(modes)>1:
        print("Attention : %s modes ! -> %s "%(len(modes), modes))

    if return_first_mode:
        return modes[0]
    return modes
def find_mod_group(database, column_for_grouping, column_searched_for_mode, fill_na_with=0, create_label_column=True, create_count_column=True, create_percent_column=True):
    """
    Create a dataframe with the mode / the count of mode and the pct of mode by group of 'column_for_grouping'.
    :param database: pandas DataFrame
    :param column_for_grouping: string (must be in the dataframe columns)
    :param column_searched_for_mode: string (must be in the dataframe columns)
    :param fill_na_with: Int (value with which to replace empty values)
    :param create_label_column: Boolean (create the label column on the returned dataFrame)
    :param create_count_column: Boolean (create the count column on the returned dataFrame)
    :param create_percent_column: Boolean (create the percent column on the returned dataFrame)

    :return:
    """
    import pandas as pd
    import sys

    assert isinstance(database, pd.DataFrame)           , "df is not a DataFrame, it is : %s"%(type(database))
    assert column_for_grouping      in database.columns , "Group is not in DataFrame"
    assert column_searched_for_mode in database.columns , "Column is not in DataFrame"
    assert isinstance(fill_na_with          , int )
    assert isinstance(create_label_column   , bool)
    assert isinstance(create_count_column   , bool)
    assert isinstance(create_percent_column , bool)

    database = database.fillna(fill_na_with)

    col_count       = database[column_searched_for_mode].groupby(database[column_for_grouping]).count()
    distinct_value  = database[column_searched_for_mode].groupby(database[column_for_grouping]).agg(lambda x: len(  x.value_counts().values     )   )
    mode_value      = database[column_searched_for_mode].groupby(database[column_for_grouping]).agg(lambda x:       x.value_counts().index [0]      )
    mode_count      = database[column_searched_for_mode].groupby(database[column_for_grouping]).agg(lambda x:       x.value_counts().values[0]      )
    pct_mode        = mode_count/col_count

    mode_value.name     = "mode_%s"%(            column_searched_for_mode)
    mode_count.name     = "cnt_mode_%s"%(        column_searched_for_mode)
    pct_mode.name       = "pct_mode_%s"%(        column_searched_for_mode)
    distinct_value.name = "distinct_val_for_%s"%(column_searched_for_mode)


    mode_value      = convert_serie_to_df(mode_value    , column_for_grouping)
    mode_count      = convert_serie_to_df(mode_count    , column_for_grouping)
    pct_mode        = convert_serie_to_df(pct_mode      , column_for_grouping)
    distinct_val    = convert_serie_to_df(distinct_value, column_for_grouping)

    new_df = pd.DataFrame()
    if create_label_column:     new_df = merge_df(mode_value, mode_count    )
    if create_count_column:     new_df = merge_df(new_df    , pct_mode      )
    if create_percent_column:   new_df = merge_df(new_df    , distinct_val  )

    return new_df
def get_merge_from_file(left_filename, left_union_on, df_right, right_union_on, debug=False):
    """
    Create a DF from a path, and directly merge it with another (already created before) passed as parameter.
     Used to 'add' a 'reference column' to a file when creating a DF out of it.

    :param left_filename: string (path from which to create the new df)
    :param left_union_on: string (columns of the new df on which to make the merge)
    :param df_right: pandas.DataFrame (df from wich to extract the appended column)
    :param right_union_on: string (column on which to merge on the right dataframe)
    :param debug: boolean (print some info to screen)
    :return: pandas.DataFrame (created from the file path, and extended from the union with the right DF)
    """
    import pandas as pd
    import os
    """
        Getinng DF
    """

    df_left = get_dataframe(left_filename, ";")
    assert left_union_on in df_left.columns, "%s not in %s" % (left_union_on, df_left.columns)
    """
        MERGING
    """
    unioned_df_left = pd.merge(df_left,
                               df_right,
                               left_on=left_union_on,
                               right_on=right_union_on,
                               how='left');
    """
        Verify
    """
    title2("Merged %s :" % (left_filename))
    if debug:
        print(unioned_df_left.ix[unioned_df_left[analysis_level].notnull(), :])
    return unioned_df_left
def get_percentage_of_a_value_in_a_col_by_group(database,col_to_group_by, colonne_to_test, searched_value, fill_na_with=0):
    """
    Create a new DataFrame containing two cols :
    (1) A col with all the modalities in col_to_gp_by
    (2) the pourcentage of [searched_value] for the [colonne_to_test] for each group

    :param database: pd.DataFrame
    :param col_to_group_by: string
    :param colonne_to_test: string
    :param searched_value: object
    :param fill_na_with: object (generally int)
    :return: pd.DataFrame : (two_columns)
    """
    import pandas as pd
    assert isinstance(database, pd.DataFrame), "database is not a pd.DataFrame, is is : %s"%(type(database))
    assert col_to_group_by in database.columns, "%s not in the database columns (%s)"%(col_to_group_by, database.collumns)
    assert colonne_to_test in database.columns, "%s not in the database columns (%s)"%(colonne_to_test, database.collumns)

    database   = database.fillna(fill_na_with)
    gpd_col     = database[colonne_to_test].groupby(database[col_to_group_by])
    pct_by_gp = gpd_col.agg(test, wanted_value=searched_value)
    new_col_name = "pct_%s_in_%s"%(searched_value, colonne_to_test)
    pct_by_gp.name=new_col_name
    return convert_serie_to_df(pct_by_gp, col_to_group_by)
def get_delay_between_to_dates_by_group(database, col_to_group_by, col_date_1, col_date_2):
    """

    :param database:
    :param col_to_group_by:
    :param col_date_1:
    :param col_date_2:
    :return:
    """
    import pandas as pd

    name ="delay_%s_moins_%s"%(col_date_2, col_date_1)

    col_1 = pd.to_datetime(database[col_date_1])
    col_2  = pd.to_datetime(database[col_date_2])


    delay = pd.DataFrame()
    delay [col_to_group_by]=database[col_to_group_by]
    delay [name]=col_2 - col_1
    modes_delay = find_mod_group(delay , col_to_group_by, name)
    print("modes_delay.shape = ", modes_delay.shape)
    return modes_delay
def get_percentage_of_a_value_in_a_col(dataframe, colonne, searched_value) -> float:
    """
    Return the % that a value represent in a column.

    :param dataframe:
    :param colonne:
    :param searched_value:
    :return:
    """
    serie  = dataframe[colonne]
    mask   = serie==searched_value
    nb_ok  = mask.value_counts()[True]
    pct_ok = nb_ok / len(mask)
    return pct_ok
def get_pourcentages_by_col(df):
    """
    Retourne une dataframe avec les calculs des % d'unique, de null et de non null
    dans la df passée en paramètre.
    """
    df1 = calculate_unique_null_not_null(df)
    return get_pct_columns(df1)
def get_pct_columns(df, substr_pour_pct="pct"):
    """
    retourne les colonnes comportant le mot clef "pct".
    """
    return df[select_columns_by_substr(df, substr_pour_pct)]
def merge_df(df1, df2, how='left', debug=False):
    """
    Merge 2 dataframe, and display the change in size.
    :param df1: pandas dataframe
    :param df2: pandas dataframe
    :return: datframe
    """

    import pandas as pd
    assert isinstance(df1, pd.DataFrame), "DF1 is not a Pandas DataFrame, it is a [%s]"%(type(df1))
    assert isinstance(df2, pd.DataFrame), "DF2 is not a Pandas DataFrame, it is a [%s]"%(type(df2))
    how_possibilities =['left', 'right', 'outer', 'inner']
    assert how in how_possibilities , "Parameter [how] is not in %s"%(how_possibilities )
    for i in set(df1.columns).intersection( set(df2.columns)):
        pass

    new_df = pd.merge(df2,df1, how="right")#, on=i)

    assert new_df.shape[0]==df1.shape[0], "Merged from %s & %s to %s.  "%(df1.shape, df2.shape, new_df.shape)
    if debug:
        print("Merging From %s / %s to %s : %s "%(df1.shape, df2.shape, new_df.shape, new_df.columns))
    return new_df
def merge_df_array(array_of_dataframes):
    """

    :param array_of_dataframes:
    :return:
    """
    import pandas as pd

    assert all([True if isinstance(x, pd.DataFrame) else False for x in array_of_dataframes ])," a parameter is not a DF : %s "%([type(x) for x in array_of_dataframes])
    merged_df = array_of_dataframes[0]
    for dataframe in array_of_dataframes[1:]:
        merged_df  = merge_df(merged_df , dataframe)
    return merged_df
def merge_on(df1, df2, col_df1, col_df2, how='left', debug=False):
    """
    Merge 2 dataframe, and display the change in size.
    :param df1: pandas dataframe
    :param df2: pandas dataframe
    :return: datframe
    """

    import pandas as pd
    assert isinstance(df1, pd.DataFrame), "DF1 is not a Pandas DataFrame"
    assert isinstance(df2, pd.DataFrame), "DF2 is not a Pandas DataFrame"
    try:
        unioned_df_left= pd.merge(df1                   ,
                                  df2                   ,
                                  left_on   = col_df1   ,
                                  right_on  = col_df2   ,
                                  how       = how       )
        return unioned_df_left
    except Exception as e:
        print("!! %s"%(e))
def plot_column(df, col_name, save_path=False):
    """
    Not tested
    :param df:
    :param col_name:
    :param save_path:
    :return:
    """
    import matplotlib.pyplot as plt
    h_size  = 2.5
    w_size  = 3.5
    n_row   = 1
    n_col   = 3
    axe_nb  = 0
    color1  = 'pink'
    color2  ='orange'
    print(col_name)
    fig     = plt.figure( figsize=(w_size*n_col,h_size*n_row))
    fig.subplots_adjust(hspace=0.4, wspace=0.6)
    # distribution
    axe_nb +=1
    ax1 = fig.add_subplot(n_row,n_col,axe_nb)
    ax1.set_xlabel('Satisfaction (10 = best)')
    try:
        df[col_name].plot(kind='hist', color='pink', title="%s distribution"%col_name)
    except:
        print('pbm with df[col_name].plot(kind=hist, color=pink, title= s distribution col_name) : col_name = %s'%(col_name))
    #
    axe_nb +=1
    ax2 = fig.add_subplot(n_row,n_col,axe_nb)
    ax2.set_xlabel('Satisfaction (10 = best)')
    ax1.set_ylabel("Nb of responses")
    ax2.set_ylabel("Nb of responses")
    df[col_name].plot(kind='box', color='pink', title="%s distribution"%col_name)
    #
    axe_nb +=1
    ax3 = fig.add_subplot(n_row,n_col,axe_nb)
    y_max = df.groupby(lambda x: (x.year, x.month))[col_name].mean().max() +1
    y_min = df.groupby(lambda x: (x.year, x.month))[col_name].mean().min() -1
    ax3.set_ylim(y_min,y_max)
    ax3.plot(df.groupby(lambda x: (x.year, x.month))[col_name].mean(), color=color1)
    ax3.set_title("%s and Nb of answers"%col_name)
    ax3.set_ylabel("Mensual %s Mean (%s)"%(col_name, color1))
    ax3.set_xlabel("Month")
    ax4 = ax3.twinx()
    ax4.plot(df.groupby(lambda x: (x.year, x.month))[col_name].count(),  color=color2)
    ax4.set_ylabel("Mensual Nb of answers (%s)"%(color2))

    if save_path:
        fig.savefig(save_path)

    """
    ax5 = fig.add_subplot(2,2,4)
    ax5.hist(date_speedy.groupby(lambda x: (x.year, x.month))["Q7"].count(), color='pink')
    ax5.set_title("Q7 and Nb of answers")
    ax5.set_ylabel("Mensual Q7 Mean (pink)")
    """
def plot_pct_columns(df):
    """
    plot un graph des colonnes pourcentage.
    """
    df[select_columns_by_substr(df, "pct")].plot(kind="barh")
def select_items_beginning_with(array, searched_substring):
    """
    Search a substing in the array, and return the array containing the substring.

    :param array: where to search
    :param substring: searched
    :return: array
    """
    return [x for x in array if x.lower().startswith(searched_substring.lower())]
def split_file_on_date(path_to_file, date_column, output_dir,  sep=';', chunksize=100000):

    import pandas as pd
    assert is_file(path_to_file), "%s is not a file !"%(path)
    assert isinstance(date_column, str), "%s should be a str"%(date_column)
    if not output_dir.endswith('/'):
        output_dir = output_dir + '/'
    csv_reader = pd.read_csv(path_to_file, chunksize=chunksize, sep=sep, error_bad_lines=False)
    header_written = list()
    n = 0
    title("Reading %s"%(path_to_file))
    for df in csv_reader:
        print(df[date_column])
        df['year'] = df[date_column].apply(lambda x : x[:4] if isinstance(x, str) else "-1")

        for y, v in df.groupby('year'):
            output_path = "%s%s_%s.csv"%(output_dir, get_file_name(path_to_file), str(y))
            if y not in header_written:
                title2("Beginning new file : %s" % (output_path ))
                v.to_csv(output_path , mode='a', header=True, index=False)
                header_written.append(y)
            else:
                v.to_csv(output_path , mode='a', header=False, index=False)
            n+=chunksize
            print("Read %s lines"%n)

    print("%s lines written for years %s"%(n, header_written))
def title(string, timeinfo=time.time()):
    """
    Print a string to the screen, with a timestamp
    :param string:
    :param timeinfo:
    :return:
    """
    import time
    blank           = " "*int(((30-len(string)))/2)
    time_           = "(%s)"%(time.time() - timeinfo)
    title           = blank , string, time
    horizontal_line = "*"*30
    print(horizontal_line)
    print(title)
    print(horizontal_line)
    return time.time()
def title2(string, timeinfo=time.time()):
    import time
    sep = "-"
    n = 40
    if "end" not in string:
        print(sep*n)
    print(" "*int(((n-len(string))/2)), string, "(", "%s"%(time.time() - timeinfo), ")")
    if "end" in string:
        print(sep*n)

    return time.time()
def test(array, wanted_value):
    """
    Create an array of one for each elem in array being equals to the wanted value.
    Called by group.agg
    :param array:  array where to search for the value.
    :param wanted_value: value searched within the array.
    :return: float len(original array)
    """

    if len(array)==0:
        return 0
    nb_val = len(array)
    oks = [1 for x in array if x==wanted_value]
    nb_ok = len(oks)
    return nb_ok/nb_val
def try_to_transform_string_to_float(string):
    try:
        return float(string)
    except:
        try:
            return float(string.replace(",", "."))
        except:
            return string
def test_date(x, format, erreurs, ):
    import datetime
    try:
        return datetime.datetime.strptime(x, format)
    except:
        for erreur in erreurs:
            if erreur in x:
                return None 
        raise
def select_columns_by_substr(df, substr):
    """
    Renvoie la liste des colonnes comprenant une chaine de caractères.
    """
    return [col for col in df.columns if substr in col]
def reject_outliers(data, m = 2.):
    import numpy as np
    d = np.abs(data - np.median(data))
    mdev = np.median(d)
    s = d/mdev if mdev else 0.
    return data[s<m]
def get_outliers(data, m = 2.):
    import numpy as np
    d = np.abs(data - np.median(data))
    mdev = np.median(d)
    s = d/mdev if mdev else 0.
    return data[s>m]
def frange(start, stop, step):
    """
    Create an array containing numbers from [start] to [stop] with a step of [step]

    Parameters:
        start: int
        stop : int
        step : int

    Return:
        r : array of int
    """
    r = [start]
    i = start
    while i<stop:
        i+= step
        r.extend([i])
    return r



class path_cleaner:
    """
    Class which rename files so that they are compatible with OneDrive
    """
    unwanted_chars_in_directories = ["\\", ":", "*", "?", "\"", ">", "<", "|", "�"]
    unwanted_chars_in_filenames = ["/", "\\", ":", "*", "?", "\"", ">", "<", "|", "�"]    
    def remove(string, list_of_unwanted_car, replacement_char="_"):
        """
        Remove unwanted chars from a string.
        """
        new_string = string
        for unwanted_char in list_of_unwanted_car:
            new_string = new_string.replace(unwanted_char, replacement_char)
        return new_string

    def clean_dirname(self, dirname):
        """
        Remove unwanted characters from directory names.
        """
        return remove(dirname,self.unwanted_chars_in_directories)

    def clean_filename(self, filename):
        """
        Remove unwanted characters from filename.
        """
        return remove(filename,self.unwanted_chars_in_filenames)

    def ensure_compatible_directory_name(self, actual_dirname):
        """
        Rename a the directory if it has an unwanted character.
        """
        cleaned_dirname = clean_dirname(actual_dirname)
        if actual_dirname != cleaned_dirname:
            import os
            os.rename(actual_dirname, cleaned_dirname)
            print(("rename = %s to %s"% (actual_dirname, cleaned_dirname)))
        return cleaned_dirname


    def make_sub_directories_compatible_with_oneDrive(self, directory_path):
        """
        Rename all subdirectory so that they are compatible with oneDrive.
        """
        assert os.path.isdir(directory_path), "Dir [{0}] doesn't exist".format(directory_path)
        for dirname, dirnames, filenames in os.walk(directory_path) :
            dirnames = [os.path.join(dirname,sub_directory) for sub_directory in dirnames]
            list(map(ensure_compatible_dirname, dirnames))



# -*- coding: utf-8 -*-
"""
Functions to help manipulating files.
"""
__author__ = 'romain'

def is_dir(dirpath):
    """
    Check either a path is a directory.
    :param dirpath: path to check
    :return:Boolean
    """
    return path_is_dir(dirpath)
def is_file(path):
    """
    Check either the given path is a file
    :param path: path to check
    :return: Boolean
    """
    return path_is_file(path)
def clean_dir_filenames(dirpath):
    """
    Call clean_filenames on all the file under the given dirpath
    :param dirpath: path to a directory
    :return:nothing
    """
    assert is_dir(dirpath)
    for path in get_filepaths(dirpath):
        clean_file_name(path)
def clean_file_name(path):
    """
    Replacce whites and 'minus' from filename to "_".
    :param path: file path
    :return: nothing (changing the names on the filesystem)
    """
    assert is_file(path)
    if path_is_not_a_dir(path):
        import ntpath, os
        dir = ntpath.dirname(path)
        old_path = os.path.join(dir , ntpath.basename(path))
        new_path = os.path.join(dir , ntpath.basename(path).replace(" ", "_").replace("-", "_").lower())
        if new_path != old_path :
            import os
            print("moving %s \n to: %s"%(old_path, new_path))
            os.rename(path, new_path)
        print("%s \n equals \n%s\n" %(new_path, old_path))
def clean_file_names(paths):
    """
    !! untested !!

    Remove ["_","(", ")"] and lowercase filenames.

    Parameters:
        paths : array of paths
    Return :
        Nothing : change filename on the file system (+ print on stdout)
    """
    import ntpath
    for path in paths:
        dir_ = ntpath.dirname(path)
        filename = ntpath.basename(path)
        clean_filename = filename.lower().replace(" ", "_").replace("(", "").replace(")", "").replace("_-_", "_")
        clean_path = dir_ + "/" + clean_filename
        print("trying to os.rename(%s,%s) " % (path, clean_path));
        os.stdout.flush()
        os.rename(path, clean_path)
def path_is_file(path):
    """
    Return True is path is a file, Raise an exception otherwise.
    """
    import os
    #assert(os.path.exists(path)), "'Path' parameter is not a file (%s)"%(path)
    return os.path.exists(path)
def path_is_not_a_file(path):
    """
    Return True is path is not a file, Raise an exception otherwise.
    """
    import os
    assert(not os.path.exists(path)), "'Path' parameter is an existing file (%s)"%(path)
    return True
def path_is_dir(directory):
    """
    Return True is path is a directory, Raise an exception otherwise.
    """

    import os
    assert(os.path.isdir(directory)), "The 'directory' parameter is not a directory (%s)"%(directory)
    return True
def path_is_not_a_dir(directory):
    """
    Return True is path is not a directory, Raise an exception otherwise.
    """
    import os
    assert(not os.path.isdir(directory)), "The 'directory' parameter is a directory (%s)"%(directory)
    return True
def do_extract(path, n_ligne, output_dir="./", debug=True):
    """
    Create a new file and write on it the n first ligne of the given path.
    :param path: file from which to extract lines
    :param n_ligne: number of ligne to extract (is used on the new-file naming)
    :param output_dir: if we want to write on a specific dir
    :return: nothing (new file created)
    #TODO: change to a panda trunk reading ?
    """
    import os
    assert path_is_file(path)
    assert is_dir(output_dir)
    n_written = 0
    output_path = os.path.join(output_dir , get_file_name(path)+"_%s.csv"%n_ligne)
    assert path_is_not_a_file(output_path), "%s already exist."%(output_path)
    with open(path, "r") as input_file:
        with open(output_path, "w") as output_file:
            while n_ligne > 0:
                ligne = input_file.readline()
                output_file.write(ligne)
                n_written += 1
                n_ligne = n_ligne -1
            if debug :
                print("Wrote %s lines into %s."%(n_written, output_path))
                print("Target dir = %s"%(output_dir))
def concatenate_csv(list_csv_path, path_output_file):
    """
    Write several files into the path_output_file (no header logic done !)
    :param list_csv_path: list of full path
    :param path_output_file: where to write the concatenation
    :return: void
    """
    #sanity check
    assert path_is_not_a_file(path_output_file)
    for path in list_csv_path: assert path_is_file(path)
    if not len(list_csv_path): return -1

    #var declaration

    #algo
    try:
        with open(path_output_file, "w") as outputfile:
            first_input_file= list_csv_path.pop()
            for line in open(first_input_file):
                outputfile.write(line)

            for path in list_csv_path:
                with open(path) as input_file:
                    next(input_file)
                    for line in input_file:
                        outputfile.write(line)
    except Exception as e:
        print("pbm in concatenate_csv: %s"%(e))
def get_filepaths(directory, extension=None):
    """
    Generate the file names in a directory
    tree by walking the tree either top-down or bottom-up. For each
    directory in the tree rooted at directory top (including top itself),
    it yields a 3-tuple (dirpath, dirnames, filenames).
    """
    import os, ntpath
    assert(os.path.isdir(directory)), "The 'directory' parameter is not a directory (%s)"%(directory)

    file_paths = []  # List which will store all of the full filepaths.
    # Walk the tree.
    for root, directories, files in os.walk(directory):
        for filename in files:
            # Join the two strings in order to form the full filepath.
            filepath = os.path.join(root, filename)

            if not ntpath.basename(filepath).startswith("."):
                file_paths.append(filepath)  # Add it to the list.

    if extension:return [x for  x in file_paths if x.endswith(extension) ]

    return file_paths  # Self-explanatory.
def get_header(path, delimiter):
    """

    :param path: full path to csv
    :param delimiter: csv delimiter
    :return: array of string
    """
    import os
    import csv
    assert(os.path.exists(path)), "'Path' parameter is not a file (%s)"%(path)
    try:
        result = []
        with open(path, 'rb') as csvfile:
            csvreader = csv.reader(csvfile, delimiter=delimiter)
            for row in csvreader:
                result = [r for r in row]
                break
        return result
    except:
        print("pbm")
        return False
def get_headers(csv_path, sep=";"):
    """
    Call a sub function with distinct encoding parameter to try reading csv.
    :param csv_path: complete path to csv file
    :param sep: csv separator
    :return: array
    """
    assert is_file(csv_path)
    r = False
    try:
        r = get_headers_sub(csv_path, ";", 'utf-8')
    except:
        pass
        #print "not utf-8"
    if not r:
        try:
            r = get_headers_sub(csv_path, ";", 'latin-1')
        except:
            pass
            #print "not latin-1"
    return r
def get_headers_sub(csv_path, sep=";", encoding_='utf-8'):
    """
    Try opening a csv file with a given encoding to read the first line.
    :param csv_path: complete path
    :param sep: csv separator
    :param encoding_: encoding
    :return: array
    """
    assert is_file(csv_path)
    from pandas import read_csv
    csv_reader = read_csv(csv_path, sep, iterator=True, chunksize=10, encoding=encoding_ )
    try:
        for chunk in csv_reader:
            r = []
            for row in chunk:
                r.append(row)
            return r
            break

    except Exception as e:
        #print "pbm reading the csv : %s"%(e)
        return False
def get_all_csvs_underdir(directory_path):
    """
        Call [get_all_specific_paths_under_dir] leeting wanted_ends to its default value.

        Return :
            (Array of paths)
    """
    return get_all_specific_paths_under_dir(directory_path)
def get_all_specific_paths_under_dir(directory_path, wanted_end=".csv"):
    """
        Recursevely walk a directory and create an array of path leading to a csv file.

        Parameter:
            directory:string
                Path to the root directory to walk through.
            wanted_ends:string
                Ending we are looking for into the filename. (ie: '.csv' for csv files)
                Default = ".csv" (ie returning list of csv files)int
        Return: array
            Array of paths (csv as defaults).
    """
    import os
    files_paths = []
    for dirname, dirnames, filenames in os.walk(directory_path):
        for filename in filenames:
            if filename.endswith(wanted_end):
                files_paths.append(os.path.join(dirname, filename))

    return files_paths
def get_file_name(path):
    """
    Return jsut the filename (no path, no extension).
    :param path: complete file path
    :return: filename without extension
    """
    assert is_file(path)
    import ntpath
    import os
    base = os.path.basename(path)
    return "".join(os.path.splitext(base)[0:2])
def get_containing_dir(path):
    """
    Return the path of the containing dir
    :param path: complete file path
    :return: str
    """
    return  get_file_name_infos(path)[0]
def get_file_name_infos(path):
    """
    :param path: complete file path
    :return: infos
    """
    assert is_file(path)
    import ntpath
    import os
    base                 = os.path.basename(path)
    (filename, filetype) = os.path.splitext(base)
    dirname              = os.path.dirname (path)
    return  (dirname, filename, filetype)
def get_txt_and_csv(dir_path, txt_sep=";", csv_sep=";"):
    txt= get_filepaths(dir_path, ".txt" )
    csv= get_filepaths(dir_path, ".csv" )
    return txt+csv
def remove_string_from_file(path, to_search, replace_by=""):
    return replace_string_from_file(path, to_search, replace_by="")
def replace_string_from_file(path, to_search, replace_by=""):
    """
    Replace a string (or a list of strings) by another one.
    :param path: file to be changed
    :param to_search: string or sring array to be changed
    :param replace_by: string to write instead of the string to be replaced
    :return: path to the new file
    """
    assert isinstance(to_search, list) or isinstance(to_search, str), "%ss should be a list or a string"%(to_search)

    print("Going to replace %s by [%s] from %s "%(to_search, replace_by, path))

    if isinstance(to_search, str):
        to_search = [to_search]
    output_path = path+"_small"
    with open(output_path , "w") as f2:
        with open(path, "r") as f:
            for l in f:
                to_write = l
                for to_be_replaced in to_search:
                    to_write = to_write.replace(to_be_replaced, replace_by)
                f2.write(to_write)

    return output_path
def explore(file, sep=";", top = 10):
    """

    """
    import pandas as pd
    print("Exploring %s"%(get_file_name(file)))
    to_explore = pd.read_csv(file, sep=";", error_bad_lines=False)
    nb_lignes  = to_explore.shape[0]
    cols       = to_explore.columns
    i          = -1
    for col in cols:
        i       += 1
        serie    = to_explore[col]
        null     = serie.isnull()
        not_null = len([x for x in null if not x])

        print("%s) %s"%(i,col))
        print(not_null, " values not null out of ", nb_lignes, "(", not_null/nb_lignes, "), top %s :"%(top))
        s        = serie.value_counts()
        d        = s/not_null
        d        = ["%.2f"%(x) for x in d]
        top_     = list(zip(s.index[:top], s.values[:top] , d))
        print(top_,  "\n")
    print("="*50)
def extract_column_from_csv(path, columns, output_dir, sep=';', chunksize=100000, debug=False):
    import pandas as pd
    import os
    assert is_file(path), "%s is not a file !"%(path)
    output_path = os.path.join(output_dir, "_extract_col_%s_from_%s.csv"%("__".join(columns), get_file_name(path)))
    assert path_is_not_a_file(output_path), "%s already exist : work already done ?"%(output_path)
    assert isinstance(columns, list), "%s should be a list"%(columns)
    print("Extract %s from %s by chunk of %s and store into %s (input path =%s)"%(columns, get_file_name(path),chunksize, output_dir,  path))
    csv_reader = pd.read_csv(path, chunksize=100000, sep=sep, error_bad_lines = False)
    n_ligne =0
    with open(output_path, "w") as output_file:
        header_written = False
        for df in csv_reader:
            if not header_written:
                assert all([ x in df.columns for x in columns]), "%s not in DF cols (%s)"%(columns, df.columns)
                output_file.write(df.ix[:,columns].to_csv(index=False, sep=sep))
                header_written = True
            else:
                output_file.write(df.ix[:,columns].to_csv(index=False, header=False, sep=sep))
                n_ligne += chunksize
                print(n_ligne)
    print("%s lines written in %s"%(n_ligne, output_path))
def transform_csv_into_excel(path_csv):
    """

    """
    import pandas as pd
    output_path = path_csv[:-4] + ".xls"
    writer = pd.ExcelWriter(output_path)
    df = pd.read_csv(path_csv)
    df.to_excel(writer)
    writer.save()
def split_excels(path_to_excel_file, output_dir="./", n_worksheet=3):
    """
    !! Untested !!
    Create one file by worksheet of an excel file.

    Parameters :
        path_to_excel_file : string
        output_dir : string, path to ouputdir (must end by a slash)
        n_worksheet : int, number of worksheet to look for

    Return : nothing (create files on hard drive + print infos on stdout)

    """
    import pandas as pd
    import xlrd
    import sys

    path = path_to_excel_file
    short_filename = path.replace("/", "").replace(".xlsx", "").replace(".", "");
    print("Going to read : %s" % path);
    sys.stdout.flush()
    start_reading = time.time()                             			; print("Reading")                                 ; sys. stdout.flush()
    data           = pd.read_excel( path, sheetname= list(range( n_worksheet)))
    print("Read in %s"%(time.time() - start_reading)); sys.stdout.flush()
    for worksheet in list(data.
                                  keys()):
        try:
            print("Wk %s"% worksheet); sys.stdout. flush()
            output_path   = "%s%s_worksheet_%s.csv"%(output_dir, short_filename,worksheet)
            print("Saving into %s"%(output_path))
            sys.stdout.flush()
            start_saving = time.time()
            data[worksheet].to_csv ( output_path, encoding='UTF-8')
            print( "Saved in %s"%(time . time()-start_saving))
        except Exception as e:
            print( "Erreur in worksheet %s"% worksheet)
            sys.stdout.flush()
        print(e[:100])
        sys.stdout.flush()
def change_commas_to_dots_in_dir(directory_path):
    """
        Retrieve all the csv under a root dir, and apply the change_commas_to_dots function to each.

        Args:
            directory_path (str): path to the root directory

        Returns:
            nothing
    """
    paths = get_all_csvs_underdir(directory_path)
    list(map(lambda path: change_commas_to_dots(path), paths))
def change_commas_to_dots(file_path):
    """
        Apply the shell sed command to change commas to dot on a filepath.

        Returns: nothing.
    """
    from subprocess import call
    call(["sed", "-i", "s/,/\./g", file_path])
def dico_from_two_col_tsv(fsv_path, separator ="\t"):
    """
		Open a two-col tsv file and return a dictionnary made out of it (first column is a key, second a value).

		Args:
			fsv_path  (str): path to a 'formated'-separated-value file.
			separator (str): separator used to format the separated-value file.

		Algo:
			open the file, and populate a dict line by line.

		Returns:
			dict: keys are value read from the 1st col, associated value are striped value from the 2nd col.
	"""
    dico_ = {}
    with open(fsv_path) as f:
        for line in f :
            k,v = line.split(separator)
            k,v = k.strip(), v.strip()
            dico_[k] = v

    return dico_
def find_delimiter(path_to_csv_file, encoding="utf-8"):
    """
        Test delimiters ([,], [;], [\t]) and send back the one more likely to be.

        Usage :
            Useful to find the delimiter of a csv file.
    """
    sep_to_test = [",", ";", "\t", "|"]
    with open(path_to_csv_file, encoding="utf-8") as csv_file:
        for headers in csv_file:
            occurences = [len(headers.split(x)) for x in sep_to_test]
            max_       = occurences.index(max(occurences))
            return sep_to_test[max_]


def get_headers(path_to_csv_file):
    """
        Open a csv file and create a striped array of the first line splitted by auto-detected separator.

        return: array

    """
    import csv
    with open(path_to_csv_file) as csv_file: csv_reader 	= csv.reader(
                                                                     csv_file, delimiter=find_delimiter(path_to_csv_file))
    for line in csv_reader:
        return [field.strip() for field in


line]
def create_random_sample(input_dir, output_dir, nb_lines, max_line):
    """
        Find all the csv under the input_dir and create an extract for each one in the output dir.

        !! now the output dir is not taken into account (files are created in the local dir) !!

        Parameters:
            nb_lines : int
                Number of line we want the extract to have (the extract may be way smaller for a file which doesn't have that much lines)

            max_line: int
                Maximum line we want to read in the files (this could be used to limit the execution time)

        Algo:
            1) create a random array of [ len = nb_lines ] line in the range [1, max_line].
            2) open the file, read it line by line, and write out the lines found in the random array of lines.
            3) break on either nb_lines are written or max_lines are read.

        Return : nothing

    """
    import numpy as np
    onlycsv = get_all_csvs_underdir(input_dir)
    len_input_dir = len(input_dir)
    # 1) USER INFO : print( "-"*50)print( " "* 20, "Creating random extract")
    print("-" * 50)
    print("input dir = %s" % (input_dir))
    print("%s files : [%s]" % (len(onlycsv), ",".join(onlycsv)))
    print("output dir = %s" % (output_dir))
    print("Selecting %s lines within the %s first." % (nb_lines, max_line))
    # 2) READING FILES
    for csv_path in onlycsv:
        random_lines = np.random.randint(1, max_line, size=nb_lines)  # SELECTING RANDOM LINES
        current_line = 0
        nb_written_lines = 0
        to_print = ""
        print("Dealing with : %s " % (csv_path))
        with open(csv_path, "r") as inputs:
            output_file = csv_path[:-4] + "_random_%s_lines.csv" % (nb_lines)
            with open(output_file, "w") as output:
                for line in inputs:
                    current_line += 1

                    if current_line == 1:  # WE SAVE THE HEADERS
                        to_print = line
                        continue

                    if current_line in random_lines:  # WE WRITE OUT THE RANDOM LINES
                        to_print += line
                        nb_written_lines += 1

                    if current_line % 100000 == 0:
                        output.write(to_print)
                        to_print = ""
                        print("Line %s : wrote %s lines out of %s wanted. (%.2f pct) " % (
                            current_line, nb_written_lines, nb_lines, nb_written_lines / nb_lines))
                    if nb_written_lines >= nb_lines:
                        break
                    if current_line >= max_line:
                        break
                output.write(to_print)
def extract_n_lines(input_dir, nb_lines):
    """
        Find all the csv under the input_dir and create an extract for each one.
    """

    for csv_path in get_all_csvs_underdir(input_dir):
        current_line = 0
        print("Dealing with : %s " % (csv_path))

        with open(csv_path, "r") as inputs:
            output_file = csv_path[:-4] + "_extract_%s.csv" % (nb_lines)
            with open(output_file, "w") as output:
                for line in inputs:
                    output.write(line)
                    current_line += 1
                    if current_line == nb_lines:
                        break
def count_separator(file_path, delimiter=','):
    """
        Read a file, line by line, for each line, fetch the nb of delimiter.
        If the nb of delimiter change, print a message and the line number
    """
    import sys
    num_line = 0
    last_nb_delimiter = 0

    with open("%s_strange_lines.csv" % (file_path), "w") as output:
        output.write("count_separator(%s, delimiter=%s)\n" % (file_path, delimiter))
        with open(file_path, "r") as inputs:
            for line in inputs:
                num_line += 1
                nb_delimiter = line.count(delimiter)
                if nb_delimiter != last_nb_delimiter:
                    print("%s/%s," % (num_line, nb_delimiter))
                    last_nb_delimiter = nb_delimiter
                    output.write(line)
                if num_line % 500000 == 0:
                    print(num_line, " ")
                    sys.stdout.flush()
def strip_csvs(directory_path):
    """
        Take a directory, and strip the values contained in the csvs.

        PARAMETERS:
            directory_path : directory where to search for csvs (must end with an [/])

        RETURN :
            create a '_small_' file for each file in the list (on the same directory list)
    """

    from os import listdir
    from os.path import isfile, join
    """
        PARAMETERS
    """
    if not directory_path.endswith("/"):
        directory_path = "%s/" % (directory_path)
    path = directory_path

    onlycsv = [f for f in listdir(path) if isfile(join(path, f)) if f.endswith(".csv")]
    # onlycsv = [	"H_OPRT_ACTE.csv"]

    separator = ";"
    unwanted_values = []
    default_value = ''

    """
        ALGO
    """
    for csv in onlycsv:
        big = path + csv
        small = big[:-4] + "_small_.csv"
        with open(big, "r") as too_big:
            with open(small, "w") as smaller:
                print("Reducing :\n %s\n to:\n %s " % (too_big, smaller))
                for line in too_big:
                    chunks = [x.strip() for x in line.split(separator)]
                    # chunks = [ x if x not in unwanted_values else default_value for x in chunks ]
                    smaller.write(separator.join(chunks))
                    smaller.write("\n")
                print("--")
def find_common_key(path, onlycsv):
    import csv
    """
        (2) populate a dictionnary with a table of fields by csv
            +
            populate 2 identical sets with the distincts fields values
    """
    fields = set()
    commonfields = set()
    joignable_csv = []

    for file_name in onlycsv:
        with open(path + file_name) as input_file:
            csv_reader = csv.reader(input_file, delimiter=find_delimiter(path + file_name))
            if debug: print("%s delimiter = %s " % (file_name, find_delimiter(path + file_name)))
            for line in csv_reader:
                headers[file_name] = [field.strip() for field in line]
                if debug: print("headers = %s" % (headers))
                for field in headers[file_name]:
                    fields.add(field)
                    commonfields.add(field)
                    nb_file_by_key[field] += 1
                break

    """
        (3) reduce the [commonfields] set by discarding all fields wich are not in all files
    """

    for file_name in headers:
        for field in fields:
            if field not in headers[file_name]:
                commonfields.discard(field)
    """
        (4) populate an array of files all the having at least one field in the list of commonfields
    """

    for file_name in headers:
        joignable = False
        for key in headers[file_name]:
            if key in commonfields: joignable = True

        if joignable: joignable_csv.append(file_name)

    """
        5) We select one commonkey
    """
    print("=" * 50)
    print("Looking for common key in : %s " % (onlycsv))
    commonkey = False

    try:
        commonkey = next(iter(commonfields))
    except :
        print("no commonkey")
        best_key = [(k ,v) for (k, v) in list(nb_file_by_key.items()) if v == max(nb_file_by_key.values())][0]
        print("best key : %s over %s files " % (best_key[0], best_key[1]))
        commonkey = best_key[0]
        joignable_csv = []
        for file_name in headers:
            joignable = False
            for key in headers[file_name]:
                if key == commonkey: joignable = True

            if joignable: joignable_csv.append(file_name)

    print("joignable_csv with this key : \n  |--> %s" % ("\n  |--> ".join(joignable_csv)))
    print("commonfields = %s" % (commonfields))
    print("choosen key --> %s" % (commonkey))
    print("=" * 50)
    return commonkey, joignable_csv
def split_excels(path_to_excel_file, output_dir="./", n_worksheet=3):
    """
    Create one file by worksheet of an excel file.

    Parameters :
        path_to_excel_file : string
        output_dir : string, path to ouputdir (must end by a slash)
        n_worksheet : int, number of worksheet to look for

    Return : nothing (create files on hard drive + print infos on stdout)

    """
    import pandas as pd
    import xlrd
    import sys
    import time
    from . import functions_de_decouverte_de_fichiers
    path = path_to_excel_file
    short_filename = functions_de_decouverte_de_fichiers.get_file_name(path) + ".csv";
    print("Going to read : %s" % path);
    sys.stdout.flush()
    start_reading = time.time();
    print("Reading");
    sys.stdout.flush()
    data = pd.read_excel(path, sheetname=list(range(n_worksheet)));
    print("Read in %s" % (time.time() - start_reading));
    sys.stdout.flush()
    for worksheet in list(data.keys()):
        try:
            print("Wk %s" % worksheet)
            output_path = "%s%s_worksheet_%s.csv" % (output_dir, short_filename, worksheet);
            start_saving = time.time();
            print("Saving into %s" % (output_path));
            sys.stdout.flush()
            data[worksheet].to_csv(output_path, encoding='UTF-8');
            print("Saved in %s" % (time.time() - start_saving))
        except Exception as e:
            print("Erreur in worksheet %s" % worksheet);
            sys.stdout.flush()
            print(e[:100]);
            sys.stdout.flush()
def fetch_csv_headers(dirpath="."):
    """
    Fetch all the csvs in the directory 'dirpath',
    and create a string with the name of the file, folowwed by the list of headers.

    Return:
        array of lines.
    """
    lines = []
    csv = get_all_csvs_underdir(dirpath)
    for c in csv:
        headers = get_headers(c)
        for h in headers:
            l = ntpath.basename(c) + "," + h.replace("\n", " ")
            lines.append(l)
    return lines
def dico_from_directory(dir_path):
    """
    Display on screen a python code listing the csvs in a directory on a dictionnary structure : keys = csv name, v= csv path.

    Params:
        dir_path : string
            path to the directory from which we want to display the csvs.

    Return: nothing (display on stdout)

    Usage :
        It can be useful to map a list of csv name to variables.
        A dict is a good structure for that.
        this script help coding that by displaying on stdout a dict-like frame to copy-paste in your code.

    """
    import ntpath
    csv_paths = get_all_csvs_underdir(dir_path)
    print("files = {")
    for path in csv_paths:
        print("'':'%s'," % (ntpath.basename(path)))
    print("}")
def get_directory_paths(directory, extension=None):
    """
    Generate the directory names in a directory
    tree by walking the tree either top-down or bottom-up. For each
    directory in the tree rooted at directory top (including top itself),
    it yields a 3-tuple (dirpath, dirnames, filenames).
    """
    import os, ntpath
    assert(os.path.isdir(directory)), "The 'directory' parameter is not a directory (%s)"%(directory)

    directory_paths = []  # List which will store all of the full filepaths.
    # Walk the tree.
    for root, directories, files in os.walk(directory):
        for directory in directories:
            # Join the two strings in order to form the full filepath.
            directory = os.path.join(root, directory)

            if not ntpath.basename(directory).startswith("."):
                directory_paths.append(directory)  # Add it to the list.

    if extension:return [x for  x in directory_paths if x.endswith(extension) ]

    return directory_paths  # Self-explanatory.



# -*- coding: utf-8 -*-
def has_been_checked(adresse):
    """
    verifie si un fichier en .txt a déjà été enregistré pour cette addresse
    retourne booléen
    """
    import os.path    
    return os.path.isfile("./"+adresse+".txt")
def save_adresse(data_json, adresse):
    """
    crée un fichier .txt avec le json encodé dedans
    retourne le json
    """
    import json
    with open(adresse+".txt", "w") as f:
        json.dump(data_json, f)
    return data_json
def get_adresse(adresse):
    """
    load le json associé à l'adresse
    et le retourne
    """
    assert has_been_checked(adresse)
    import json
    with open(adresse+".txt", "r") as f:
        return json.load(f)
def get_gmap_json(adresse):
    """
    récupère les infos sur une adresse, soit auprès de google, soit dans le système de fichier local
    """
    import urllib.request, urllib.parse, urllib.error
    import json 
    try:
        adresse  = urllib.parse.quote_plus(adresse)
        if has_been_checked(adresse):
            print(" . ", end=' ') 
            return get_adresse(adresse)
        else:       
            print(" GG ", end=' ')
            key      = "&key=AIzaSyBxELVjNecufswtilfW8XqOmj-fJLuUiiA"
            serveur  = "https://maps.googleapis.com/maps/api/geocode/json?address="
            
            url      = serveur+adresse+key
            response = urllib.request.urlopen(url)
            return save_adresse(json.loads(response.read()), adresse)
    except Exception as e :
        print("pbm [%s] sur [%s]"%(e, adresse))
        raise e
def lat_lon(adresse):
    """
    extrait la latitude et la longitude du json récupéré
    """
    
    try:
        data = get_gmap_json(adresse)
        lat = data ['results'][0]['geometry']['location']['lat']
        lon = data ['results'][0]['geometry']['location']['lng']
    except Exception as e:
        print("\n", e, "for [%s]"%adresse)
        return -1,-1

    return lat, lon
def get_lat(adresse):
    return lat_lon(adresse)[0]
def get_lon(adresse):
    return lat_lon(adresse)[1]


#-*- coding: utf-8 -*-
"""
	Fichiers
"""
"""
	Stats
"""
def find_type(string):
	"""
		Test the data type contained inside a string (int, float, chararray). 

		Returns (str): string's type
	"""
	from datetime import datetime
	tests = [ 	("int"			, int	), 
				("float"		, float	), 
				("chararray" 	, lambda value : datetime.strptime(value, "%d/%m/%y")),
				("chararray" 	, lambda value : datetime.strptime(value, "%Y/%m/%d")),
				("chararray" 	, lambda value : datetime.strptime(value, "%d/%m/%Y")),
				("chararray" 	, lambda value : datetime.strptime(value, "%Y-%m-%d")),
				("chararray" 	, lambda value : datetime.strptime(value, "%d-%m-%d")),
				("chararray" 	, lambda value : datetime.strptime(value, "%y/%m/%d"))]
	"""
	("date-dmy" 	, lambda value : datetime.strptime(value, "%d/%m/%y")),
	("date-Ymd" 	, lambda value : datetime.strptime(value, "%Y/%m/%d")),
	("date-dmY" 	, lambda value : datetime.strptime(value, "%d/%m/%Y")),
	("date-Y-m-d" 	, lambda value : datetime.strptime(value, "%Y-%m-%d")),
	("date-d-m-d" 	, lambda value : datetime.strptime(value, "%d-%m-%d")),
	("date-ymd" 	, lambda value : datetime.strptime(value, "%y/%m/%d"))
	"""
	for typ, test in tests:
		try 	: 
			test(string)
			return typ 
		except 	: pass
	
	try:
		s = string.replace(",", ".")
		float(s)
		return "float"
	except : pass

	return "chararray"
def detect_column_type(path_to_csv_file):
	"""
	
	"""
	# -- imports --
	import 	csv
	from 	collections import defaultdict
	# -- parameters --
	nb_line_to_test 					= 10000
	nb_of_wished_tested_value_by_col 	= 100
	# -- algo --
	with open(path_to_csv_file) as csv_file:
		csv_reader 		= csv.reader(csv_file, delimiter=find_delimiter(path_to_csv_file))
		current_line 	= 0
		type_by_header 	= {}

		for line in csv_reader:
			current_line +=1
			if current_line == 1:
				#catch headers
				headers 	= [field.strip() for field in line]
				nb_headers 	= list(range(len(headers)))
				nb_test_by_col 	= [0] * len(headers)
				for header in headers : type_by_header[header] = defaultdict(int)
			else:
				#deal with lines
				values = [field.strip() for field in line]
				for index in nb_headers:
					value 		= values[index]
					if value != "":
						colonne 	= headers[index]
						type_cell 	= find_type(value)
						type_by_header[colonne][type_cell] += 1	
						nb_test_by_col[index] +=1
				
				
				if all(nb_test_by_col[i] > nb_of_wished_tested_value_by_col for i in nb_headers) or current_line > nb_line_to_test :
					break
				
				if current_line > nb_line_to_test :
					break
	# clean results:
	for header, types in list(type_by_header.items()):
		if len(types) ==1:
			type_by_header[header] = list(types.keys())[0]
		else:
			if 	 "text" 	in list(types.keys())	: type_by_header[header] =  "text"
			elif "float" 	in list(types.keys())	: type_by_header[header] = "float"
			elif "int" 		in list(types.keys())	: type_by_header[header] = "int"
			else 							: type_by_header[header] =  "unknown ( %s)"% list(types.keys())


	return type_by_header
def help_change_col_name(df):
    """
    
    When we have a dataframe with strange column names, 
    it may be useful to create a directory with keys being simpler, and values pointing to these
    strange columns name.
    This print a pseudo code for creating such a dictionnary :
    the values are outputed to the screen, so that it can be copy-pasted into
    a script, and new key values can be implemented.
    
    Return:
        Nothing (output on stdout).
    
    
    """
    print("cols={")
    for c in df.columns:
        x = "''"
        print(x.ljust(25),":'%s',"%c)
    print('}')
def compare_two_sets(big_liste, small_liste, to_print =""):
    """
    Print on stdout some info comparing to lists of elements (interesction / difference / subset...).
    
    Parameters : 
        big_liste : left join columns
            pandas Series
        small_list : right join columns
            pandas Series
        to_print : message to print for the user
            string
    Return :
        nothing (print on stdout)

    Usage:
        It is useful to study to columns that are used as joining key to detect some
        unexpected unfitting. This function display on screen some set difference/intersection 
        calculus as first approach to join operations.
    
    """
    
    big, small                   = set(big_liste), set(small_liste)
    len_big , len_small          = len(big), len(small)
    intersection                 = big.intersection(small)
    difference                   = big.difference(small)
    big_is_subset_of_small       = big.issubset(small)
    small_is_subset_of_big       = small.issubset(big)
    elem_in_big_not_in_small     = big.difference(small)
    elem_in_small_not_in_big     = small.difference(big)
    nb_elem_in_big_not_in_small  = len(elem_in_big_not_in_small)
    nb_elem_in_small_not_in_big  = len(elem_in_small_not_in_big)
    print(to_print)
    print("len_big                       : ", len_big) 
    print("len_small                     : ", len_small)
    print("nb of intersection            : ", len(intersection))
    print("nb_elem_in_big_not_in_small   : ", nb_elem_in_big_not_in_small)
    print("nb_elem_in_small_not_in_big   : ", nb_elem_in_small_not_in_big)
    print("big_is_subset_of_small        : ", big_is_subset_of_small)
    print("small_is_subset_of_big        :  ", small_is_subset_of_big)
    print("ex 5 elem_in_big_not_in_small : ", list(elem_in_big_not_in_small)[:5])
    print("ex 5 elem_in_small_not_in_big : ", list(elem_in_small_not_in_big)[:5])
def white_rotate():
    """
    prend la figure matplotlib en cours de création et met son fond en blanc, et affiche la figure.
    Uasage :
        à appeler après un "plot" sur une dataframe
    """
    import matplotlib.pyplot as plt
    bgcolor             = 'white'
    rotation            = 50
    loc                 = 'best'
    bbox_to_anchor      = (1.05, 1)
    loc                 = 2
    borderaxespad       = 0.
    rotation            = 45
    figsize             = (3, 22)
    date_format         = '%a'
    ax                  = plt.gca()
    short_labels        = [ item.get_text()[:10]  for item in ax.get_xticklabels()   ]
    xtl                 = [ n if i%10==0 else " " for i,n  in enumerate(short_labels)]
    

    handles, labels     = ax.get_legend_handles_labels()
    #ax.set_xticklabels(xtl)
    ax.legend(handles                          , 
              labels                           ,  
              bbox_to_anchor = bbox_to_anchor  , 
              loc            = loc             , 
              borderaxespad  = borderaxespad   )
    ax.set_axis_bgcolor(bgcolor)
    #ax.xaxis_date()
    
    #ax.xaxis.set_minor_formatter(matplotlib.dates.DateFormatter(date_format))
    plt.xticks(rotation = rotation)        
    #plt.legend(loc      = loc     )
    plt.show()
    print() 




# coding: utf-8
import pandas as pd


class extra(pd.DataFrame):
    def give_categorical_columns(self, limite=50):
        """
        Retourne la liste des colonnes de la df qui contient moins de 'limite' modalités.

        Parameters :
            limit : int, nombre de modalité limite (exclusif) que doivent avoir les colonnes retournées

        Return:
            Array of strings : each string is a column name

        """
        return [col for col in self.columns if len(self[col].unique()) < limite and "DT_" not in col]

    def histo(self, cols):
        """
        S'appuie sur la df contenue dans l'objet actuel
        Pour chaque colonne :
            \-> Compte le nombre de modalité dans chacune des colonnes
                \-> Crée :
                    \-> un graphique en bar avec ce compte par modalité
                    \-> des graphiques de zoom s'il y a assez de modalités :
        Parameters:
            cols : array of string
                \-> chaque string est un nom de colonne du tableau

        """
        import matplotlib
        import matplotlib.pyplot as plt
        matplotlib.style.use('ggplot')
        for col in cols:
            try:
                count_ = self.groupby(col)[col].count()
                count_.sort_values(inplace=True, ascending=False)
                count_.index = [" %s " % x for x in count_.index]

                for kind in ['bar', 'pie'][0:count_.shape[0] - 1]:
                    plt.figure()
                    count_.plot(kind=kind, title="%s - %s valeurs" % (col, count_.shape[0]))
                    if count_.shape[0] > 8:
                        plt.figure()
                        count_[1:5].plot(kind=kind, title="%s - valeurs 2 à 4 " % (col))
                        plt.figure()
                        count_[5:].plot(kind=kind, title="%s - valeurs 4 et plus " % (col))

            except Exception as e:
                print("Pbm sur col [%s] -> %s" % (col, e))

    def get_time_col(self):
        """
        Retourne la liste des colonnes contenant "DT_" dans leur nom.
        """
        return [col for col in self.columns if "DT_" in col]

    @staticmethod
    def timeline(path, col_date, debug=False):
        """
        Créer des graphs de séries temporelles à partir d'un csv :
            - créer une pandas df depuis le chemin
            - trouve les colonnes catégorielles
                - pour chacune
                    \-> crée un graphique pour chaque valeur de la colonne
                    \-> enregistre le graphique sur le disque
        Parameters:
            path : string, chemin vers le csv de datframe
            col_date : string, colonne de date sur laquelle se baser pour le temps
            debug : boolean

        Return: None

        """
        import os
        table = extra(pd.read_csv(path, delimiter=useful.find_delimiter(path)))
        timeline2(table, path, col_date, debug=False)

    @staticmethod
    def timeline2(df, path, col_date, debug=False):
        """
        Créer des graphs de séries temporelles à partir d'un csv :
            - créer une pandas df depuis le chemin
            - trouve les colonnes catégorielles
                - pour chacune
                    \-> crée un graphique pour chaque valeur de la colonne
                    \-> enregistre le graphique sur le disque
        Parameters:
            path : string, chemin vers le csv de datframe
            col_date : string, colonne de date sur laquelle se baser pour le temps
            debug : boolean

        Return: None

        """
        import os

        test2 = extra(df)

        modalities, binary, continuous, dates = extra_pandas.select_columns_based_on_names(test2)

        cat_col = test2.give_categorical_columns()

        # extra_pandas.enforce_date(test2, col_date, "%d%b%Y:00:00:00", debug=True)
        index = test2[col_date]
        nom_table = path[path.rfind("/") + 1:path.rfind(".")]
        for col in cat_col:
            if debug: print("col = ", col)
            categories = list(test2[col].value_counts().keys())
            if debug: print("categories = ", categories)
            for cat in categories:
                test5 = test2[col]
                test5.index = index
                test6 = pd.DataFrame(test5)
                test6["date"] = test6.index
                test6["one"] = 1
                print("cat=", cat)
                year_mini = 2013
                year_maxi = 2016
                maxi1 = test6[test6[col] == cat]
                vrai_maxi = maxi1["%s" % year_mini:"%s" % year_maxi].groupby("date").count()['one'].max()
                maxi = vrai_maxi * 1.05
                mini = vrai_maxi * 0.05

                # for year in range(2014,2017):
                try:
                    plt.figure(figsize=(20, 5))
                    n1 = test6["%s" % year_mini:"%s" % year_maxi]
                    n2 = n1[n1[col] == cat]
                    n3 = n2.groupby("date").count()
                    n4 = n3.resample('d')
                    nom_table_utf8 = to_Str(nom_table)
                    col_utf8 = to_Str(col)
                    cat_utf8 = to_Str(cat)
                    col_date_utf8 = to_Str(col_date)
                    title_ = "TABLE_[%s]_COLONNE_[%s]_CATEGORIE_[%s]_nombre_par_jour_par_date_de_[%s]" % (
                    nom_table_utf8, col_utf8, cat_utf8, col_date_utf8)
                    ax = n4["one"].sum().plot(style='o', title=title_)
                    ax.set_axis_bgcolor('w')
                    ax.set_ylim(-mini, maxi)

                    ax.plot()
                    title_ = title_.replace("/", "").replace(" ", "_")
                    savedir = "./../images/table_%s/col_%s/" % (
                    nom_table_utf8.replace("/", "").replace(" ", "_"), col_utf8.replace("/", "").replace(" ", "_"))
                    print(savedir)
                    try:
                        os.makedirs(savedir)
                    except OSError:
                        if not os.path.isdir(savedir):
                            raise
                    plt.savefig(savedir + "%s.jpg" % title_)
                    plt.show()

                    # ax.set_ylim(maxi)
                except Exception as e:
                    print("e=", e)
                    pass


